#!/usr/bin/env python
# encoding: utf-8
import json
from unittest import TestCase
from unittest.mock import patch

import os

from gitflow_api.project.angular_project import AngularProject


class TestAngularProject(TestCase):

    @staticmethod
    def get_correct_package_file():
        return 'project/mock/package.json.mock'

    @patch.object(AngularProject, '_get_version_filename', get_correct_package_file)
    def test_actual_version_with_correct_setup(self):
        angular_project = AngularProject(os.path)
        actual_version = angular_project.actual_version()
        self.assertEqual(actual_version, '1.0.0')

    @staticmethod
    def write_fake_package(data, filename):
        return data

    @patch.object(AngularProject, '_get_version_filename', get_correct_package_file)
    @patch.object(AngularProject, '_write_new_version', write_fake_package)
    def test_update_version(self):
        angular_project = AngularProject(os.path)
        new_file = angular_project.update_version('2.0.0')

        self.assertTrue(new_file["version"] == "2.0.0")



#!/usr/bin/env python
# encoding: utf-8
import os
from enum import Enum

from gitflow_api.config.config import Config
from gitflow_api.project.project_manager_strategy import ProjectManagerStrategy
from gitflow_api.utilities.git_helper import GitHelper


class VersionUtils:

    def __init__(self):
        pass

    @staticmethod
    def get_new_version(version, version_type, is_snapshot, increment):
        splitted_version = version.split('.')

        if version_type != Version.NONE:
            version_number = int(splitted_version[len(splitted_version) - version_type.value])
            splitted_version[len(splitted_version) - version_type.value] = str(
                version_number
                + (increment if increment > 0 or version_number > 0 else 0))

        if is_snapshot:
            new_version = '.'.join(splitted_version) + '-SNAPSHOT'
        else:
            new_version = '.'.join(splitted_version)

        return new_version

    @staticmethod
    def diff_version(version1, version2):
        if VersionUtils.extract_version(version1, Version.MAJOR) > VersionUtils.extract_version(version2, Version.MAJOR):
            return 1
        elif VersionUtils.extract_version(version1, Version.MAJOR) < VersionUtils.extract_version(version2, Version.MAJOR):
            return -1
        else:
            if VersionUtils.extract_version(version1, Version.MINOR) > VersionUtils.extract_version(version2, Version.MINOR):
                return 1
            elif VersionUtils.extract_version(version1, Version.MINOR) < VersionUtils.extract_version(version2, Version.MINOR):
                return -1
            else:
                if VersionUtils.extract_version(version1, Version.PATCH) > VersionUtils.extract_version(version2, Version.PATCH):
                    return 1
                elif VersionUtils.extract_version(version1, Version.PATCH) < VersionUtils.extract_version(version2, Version.PATCH):
                    return -1
                else:
                    return 0

    @staticmethod
    def extract_version(version, version_type):
        try:
            splitted_version = version.replace('-SNAPSHOT', '').split('.')
            return int(splitted_version[len(splitted_version) - version_type.value])
        except Exception as e:
            return -1

    @staticmethod
    def adjust_versions():
        config = Config()
        project = ProjectManagerStrategy.get_instance(os.getcwd())

        git = GitHelper()

        git.checkout_and_pull(config.master_branch)
        master_version = project.actual_version()

        incremented_version = VersionUtils.get_new_version(master_version, Version.MINOR, True, 1)

        new_version = '{}.{}.0'.format(VersionUtils.extract_version(incremented_version, Version.MAJOR), VersionUtils.extract_version(incremented_version, Version.MINOR))

        git.checkout_and_pull(config.staging_branch)
        project.update_version(new_version)

        git.commit_and_push_update_message(config.staging_branch, new_version)

    @staticmethod
    def format_version(format, version, project_name=''):
        formatted_version = format
        formatted_version = formatted_version.replace('{project}', project_name)
        formatted_version = formatted_version.replace('{major}', str(VersionUtils.extract_version(version, Version.MAJOR)))
        formatted_version = formatted_version.replace('{minor}', str(VersionUtils.extract_version(version, Version.MINOR)))
        formatted_version = formatted_version.replace('{patch}', str(VersionUtils.extract_version(version, Version.PATCH)))

        return formatted_version


class Version(Enum):
    NONE = 0
    PATCH = 1
    MINOR = 2
    MAJOR = 3



#!/usr/bin/env python
# encoding: utf-8
import configparser
import os

from gitflow_api.config.properties import *


class Config:
    # Branches structure
    master_branch = ''
    staging_branch = ''

    hotfix_branch = ''
    feature_branch = ''
    release_branch = ''

    version = ''

    # Issue Classification - Api Labels
    feature_labels = []
    change_labels = []
    bug_labels = []
    technical_debt_labels = []
    ignore_labels = []

    version_label_prefix = ''

    # Api Config
    api_type = None
    api_key = ''
    api_url = ''

    # Communicator Config
    communicator_type = None
    communicator_release_webhook = ''
    communicator_launch_webhook = ''

    # Changelog
    changelog_create_file = False
    changelog_path = ''

    # Extension
    extension_deploy_class = None

    # Config file instance
    config_file = None

    def __init__(self):
        self.config_file = configparser.ConfigParser()
        self.config_file.read(CONFIG_FILE)

        self._set_properties()
        self._set_api_config()
        self._set_communicator_config()
        self._set_changelog_config()

    # Method for mock tests
    def get_config_file(self):
        return self.config_file

    def _set_properties(self):
        self.master_branch = self._get_property('PROPERTIES', 'master_branch', DEFAULT_MASTER_BRANCH)
        self.staging_branch = self._get_property('PROPERTIES', 'staging_branch', DEFAULT_STAGING_BRANCH)
        self.hotfix_branch = self._get_property('PROPERTIES', 'hotfix_branch', DEFAULT_HOTFIX_BRANCH)
        self.feature_branch = self._get_property('PROPERTIES', 'feature_branch', DEFAULT_FEATURE_BRANCH)
        self.release_branch = self._get_property('PROPERTIES', 'release_branch', DEFAULT_RELEASE_BRANCH)
        self.version = self._get_property('PROPERTIES', 'version', DEFAULT_VERSION)
        self.feature_labels = self._get_property('PROPERTIES', 'feature_labels', DEFAULT_FEATURE_LABELS)
        self.change_labels = self._get_property('PROPERTIES', 'change_labels', DEFAULT_CHANGE_LABELS)
        self.bug_labels = self._get_property('PROPERTIES', 'bug_labels', DEFAULT_BUG_LABELS)
        self.technical_debt_labels = self._get_property('PROPERTIES', 'technical_debt_labels', DEFAULT_TECHNICAL_DEBT_LABELS)
        self.ignore_labels = self._get_property('PROPERTIES', 'ignore_labels', DEFAULT_IGNORE_LABELS)
        self.version_label_prefix = self._get_property('PROPERTIES', 'version_label_prefix', DEFAULT_VERSION_LABEL_PREFIX)

    def _set_api_config(self):
        self.api_type = str(self._get_config_or_environment('API', 'type', 'GITFLOW_API_TYPE', None)).lower()
        self.api_key = self._get_config_or_environment('API', 'key', 'GITFLOW_API_KEY', '')
        self.api_url = self._get_config_or_environment('API', 'url', 'GITFLOW_API_URL', '')

    def _set_changelog_config(self):
        self.changelog_create_file = bool(self._get_property('CHANGELOG', 'create_file', False))
        self.changelog_path = self._get_property('CHANGELOG', 'path', '')

    def _set_communicator_config(self):
        self.communicator_type = str(self._get_config_or_environment('COMMUNICATOR', 'type', 'GITFLOW_COMMUNICATOR_TYPE', 'None')).lower()
        self.communicator_release_webhook = str(self._get_config_or_environment('COMMUNICATOR', 'release_webhook', 'GITFLOW_COMMUNICATOR_RELEASE', ''))
        self.communicator_launch_webhook = str(self._get_config_or_environment('COMMUNICATOR', 'launch_webhook', 'GITFLOW_COMMUNICATOR_LAUNCH', ''))

    def _set_extension(self):
        self.extension_deploy_class = self._get_property('EXTENSION', 'deploy_class', None)

    def _get_config_or_environment(self, group, api_config, os_environment, default_value):
        try:
            return self.config_file[group][api_config]
        except Exception as e:
            try:
                return os.environ[os_environment]
            except Exception as ex:
                return default_value

    def _get_property(self, group, config_property, default_property):
        try:
            return self.config_file[group][config_property]
        except Exception as e:
            return default_property



#!/usr/bin/env python
# encoding: utf-8

import abc


class ProjectManager(object):
    __metaclass__ = abc.ABCMeta

    global path

    def __init__(self, path):
        self.path = str(path)

    @abc.abstractmethod
    def actual_version(self):
        return

    @abc.abstractmethod
    def update_version(self, version):
        return

    @abc.abstractmethod
    def update_dependencies_version(self):
        pass

    @abc.abstractmethod
    def dependencies(self):
        return

    @abc.abstractmethod
    def test(self):
        pass

    @abc.abstractmethod
    def deploy_local(self):
        pass

    @abc.abstractmethod
    def deploy(self):
        pass


class Dependency(object):
    groupId = ''
    artifactId = ''
    groupName = ''
    version = ''

    # The class "constructor" - It's actually an initializer
    def __init__(self, groupId, artifactId, groupName, version):
        self.groupId = str(groupId)
        self.artifactId = str(artifactId)
        self.groupName = str(groupName)
        self.version = str(version)



#!/usr/bin/env python
# encoding: utf-8
import abc


class Deploy(object):
    __metaclass__ = abc.ABCMeta

    def __init__(self, classtype):
        self._type = classtype

    @abc.abstractmethod
    def deploy(self):
        pass


#!/usr/bin/env python
# encoding: utf-8

from subprocess import check_call

from gitflow_api.deploy.deploy_strategy import DeployStrategy
from gitflow_api.project.project_manager import ProjectManager, Dependency

from xml.etree import ElementTree

import json
import os


class MavenProject(ProjectManager):
    global namespaces

    def __init__(self, path):
        self.namespaces = {'xmlns': 'http://maven.apache.org/POM/4.0.0'}
        super(MavenProject, self).__init__(path)

    def actual_version(self):
        return self._get_actual_version()

    def update_version(self, version):
        mvn_cmd = 'mvn versions:set -DnewVersion={}'.format(version)
        check_call(mvn_cmd, shell=True)

        try:
            os.remove("pom.xml.versionsBackup")
        except Exception as e:
            pass

    def update_dependencies_version(self):
        mvn_cmd = 'mvn versions:use-releases -Dmessage="update from snapshot to release" scm:checkin -f {}/pom.xml'.format(
            self.path)
        check_call(mvn_cmd, shell=True)

    def dependencies(self):
        return self._get_dependencies_from_path()

    def test(self):
        mvn_cmd = 'mvn -B clean test'
        check_call(mvn_cmd, shell=True)

    def deploy_local(self):
        mvn_cmd = 'mvn -B deploy -DskipTests'
        check_call(mvn_cmd, shell=True)

    def deploy(self):
        DeployStrategy.get_instance(default_deploy_class='gitflow_api.deploy.maven.Maven').deploy()

    def _get_dependencies_from_path(self):
        tree = ElementTree.parse(self.path + "/" + "pom.xml")
        root = tree.getroot()

        # Get dependencies for root
        dependencies = []
        dependencies.extend(self._get_snapshot_dependencies_from_xml(root))

        # Get dependencies for submodules
        modules = root.findall(".//xmlns:modules", namespaces=self.namespaces)
        for module in modules:
            moduleList = module.findall("xmlns:module", namespaces=self.namespaces)

            for moduleName in moduleList:
                tree = ElementTree.parse(self.path + "/" + moduleName.text + "/pom.xml")
                dependencies.extend(self._get_snapshot_dependencies_from_xml(tree.getroot()))

        return self._group_and_order_dependencies(dependencies)

    def _get_actual_version(self):
        tree = ElementTree.parse(self.path + "/" + "pom.xml")
        root = tree.getroot()
        return str(root.find("xmlns:version", namespaces=self.namespaces).text)

    def _get_snapshot_dependencies_from_xml(self, root):
        dependencies = []
        deps = root.findall(".//xmlns:dependency", namespaces=self.namespaces)
        for d in deps:
            group_id = d.find("xmlns:groupId", namespaces=self.namespaces)
            artifact_id = d.find("xmlns:artifactId", namespaces=self.namespaces)
            version = d.find("xmlns:version", namespaces=self.namespaces)

            if artifact_id is None \
                    or version is None \
                    or group_id is None:
                continue

            if str(version.text).find('SNAPSHOT') >= 0:
                dependency = Dependency(str(group_id.text), str(artifact_id.text), '', str(version.text))
                dependencies.append(dependency)

        return dependencies

    def _group_and_order_dependencies(self, dependencies):
        projects_group = [Dependency('br.tur.reservafacil.soa', 'adapters', 'rf-adapters', '10'),
                          Dependency('br.tur.reservafacil.soa', 'dominio', 'rf-dominio', '20'),
                          Dependency('br.tur.reservafacil.soa', 'canonico', 'rf-canonico-aereo', '20')]

        grouped_dependencies = []

        for dependency in dependencies:
            filtered_project_group = list(
                filter(lambda x: dependency.artifactId.find(x.artifactId) != -1 and dependency.groupId == x.groupId,
                       projects_group))

            if len(filtered_project_group) > 0:
                dependency.artifactId = filtered_project_group[0].groupName

            if len(list(
                    filter(lambda x: dependency.artifactId.find(x.artifactId) != -1 and dependency.groupId == x.groupId,
                           grouped_dependencies))) == 0:
                grouped_dependencies.append(dependency)

        return grouped_dependencies

    def _json_to_object(self, data):
        return json.loads(data, object_hook=lambda d: Namespace(**d))



#!/usr/bin/env python
# encoding: utf-8

from subprocess import check_call, run

from gitflow_api.deploy.deploy_strategy import DeployStrategy
from gitflow_api.deploy.pypi import Pypi
from gitflow_api.project.project_manager import ProjectManager

import re
import shutil


class PythonProject(ProjectManager):

    def __init__(self, path):
        super().__init__(path)

    def update_dependencies_version(self):
        pass

    def actual_version(self):
        lines = self._read_all_version_file()

        for line in lines:
            if str(line).lower().find('version=') >= 0:
                version = re.search('(?:(\d+)\.)?(?:(\d+)\.)?(?:(\d+)\.\d+)', str(line))
                if version is None or len(str(version.group(0)).split('.')) != 3:
                    raise ValueError('Version has incorrect format')
                return version.group(0)

        raise ValueError('Version tag not found')

    def update_version(self, version):
        lines = self._read_all_version_file()

        i = 0
        while i < len(lines):
            version_position = str(lines[i]).lower().find('version="')
            if version_position >= 0:
                lines[i] = lines[i][:version_position + 9] + str(version) + '",\n'

            i += 1

        self._write_new_version(lines, self._get_version_filename())

        return lines

    def _read_all_version_file(self):
        version_file = open(self._get_version_filename(), 'r')
        lines = version_file.readlines()
        version_file.close()
        return lines

    def dependencies(self):
        return []

    def test(self):
        # python -m unittest discover -s tests
        pass

    def deploy_local(self):
        pass

    def deploy(self):
        DeployStrategy.get_instance(default_deploy_class='gitflow_api.deploy.pypi.Pypi').deploy()

    @staticmethod
    def _get_version_filename():
        return 'setup.py'

    @staticmethod
    def _write_new_version(lines, version_filename):
        version_file = open(version_filename, 'w')
        version_file.write(''.join(lines))
        version_file.close()



#!/usr/bin/env python
# encoding: utf-8
import os

from gitflow_api.api.api_strategy import ApiStrategy
from gitflow_api.config.config import Config
from gitflow_api.utilities.git_helper import GitHelper


class Feature:

    config = None

    def feature_start(self, args):
        path = os.getcwd()
        api = ApiStrategy.get_instance(path)
        git = GitHelper()

        branch = self._get_config().feature_branch.format(args.branch)
        git.create_new_branch_from(self._get_config().staging_branch, branch)

        title = args.title if args.title is not None else branch
        issue = args.issue if args.issue is not None else 0
        merge_request = api.get_merge_request_api().create_merge_request(git.get_current_url(), branch, 'WIP: ' + title,
                                                                         '', issue,
                                                                         self._get_config().staging_branch, 'story')
        print('Branch {} and merge_request {} as created'.format(branch, merge_request.iid))

    def feature_finish(self, args):
        path = os.getcwd()
        api = ApiStrategy.get_instance(path)
        git = GitHelper()

        branch = str(git.get_current_branch() if args.branch is None else args.branch)

        try:
            api.get_merge_request_api().validate_and_close_merge_request(git.get_current_url(), branch)
            git.get_git_cmd().checkout(self._get_config().staging_branch)
            git.get_git_cmd().pull()

            print('Feature {} merged on {}'.format(branch, self._get_config().feature_branch))

        except ValueError as e:
            print(str(e))


    def _get_config(self):
        if self.config is None:
            self.config = Config()
    
        return self.config


#!/usr/bin/env python
# encoding: utf-8
import shutil
from subprocess import check_call

from gitflow_api.deploy.deploy import Deploy


class Pypi(Deploy):

    def __init__(self):
        super(Pypi, self).__init__(Pypi.__class__)
        self.__qualname__ = 'Pypi'

    def deploy(self):
        # remove dist and build files
        try:
            shutil.rmtree('dist')
            shutil.rmtree('build')
        except FileNotFoundError:
            print('Skipping deleting folder')

        cmd = 'python3 setup.py sdist bdist_wheel'
        check_call(cmd, shell=True)

        cmd = 'twine upload dist/*'
        check_call(cmd, shell=True)



#!/usr/bin/env python
# encoding: utf-8
import os

from gitflow_api.api.api_strategy import ApiStrategy
from gitflow_api.config.config import Config
from gitflow_api.utilities.git_helper import GitHelper


class Hotfix:
    config = None

    def hotfix_start(self, args):
        path = os.getcwd()
        api = ApiStrategy.get_instance(path)
        git = GitHelper()

        branch = self._get_config().hotfix_branch.format(args.branch)
        git.create_new_branch_from(self._get_config().master_branch, branch)

        title = args.title if args.title is not None else branch
        issue = args.issue if args.issue is not None else 0
        merge_request = api.get_merge_request_api().create_merge_request(git.get_current_url(), branch, 'WIP: ' + title,
                                                                         None,
                                                                         issue,
                                                                         self._get_config().master_branch, 'bug')
        print('Branch {} and mergeRequest {} as created'.format(branch, merge_request.iid))

    def hotfix_finish(self, args):
        path = os.getcwd()
        api = ApiStrategy.get_instance(path)
        git = GitHelper()

        branch = str(git.get_current_branch() if args.branch is None else args.branch)

        try:
            api.get_merge_request_api().validate_and_close_merge_request(git.get_current_url(), branch)
            git.get_git_cmd().checkout(self._get_config().master_branch)
            git.get_git_cmd().pull()

            print('Hotfix {} merged on {}'.format(branch, self._get_config().master_branch))

        except ValueError as e:
            print(str(e))

    def _get_config(self):
        if self.config is None:
            self.config = Config()

        return self.config



#!/usr/bin/env python
# encoding: utf-8

from gitflow_api.utilities.git_helper import GitHelper


class Project:
    global gl

    def __init__(self, gl):
        self.gl = gl

    def find_project_by_url(self, url):
        group_name, project_name = GitHelper.extract_group_and_project_from_url(url)
        group = self.find_group_by_name(group_name)

        return self.find_project_by_group_and_name(group, project_name)

    def find_group_by_name(self, group_name):
        groups = self.gl.groups.list(all=True)

        filtered_group = list(filter(lambda x: str(x.full_path).lower() == group_name.lower(), groups))
        if len(filtered_group) == 0:
            raise Exception('Group not found: {}'.format(group_name))
        else:
            return filtered_group[0]

    def find_project_by_group_and_name(self, group, project_name):
        projects = group.projects.list(all=True)
        project = list(filter(lambda x: x.name == project_name, projects))[0]
        return self.gl.projects.get(project.id)



#!/usr/bin/env python
# encoding: utf-8

import abc
import os


class ApiMergeRequest(object):
    __metaclass__ = abc.ABCMeta

    global path
    global gl

    def __init__(self, gl):
        self.path = os.getcwd()
        self.gl = gl

    @abc.abstractmethod
    def find_merge_request_by_url_and_branch(self, git_url, branch):
        return

    @abc.abstractmethod
    def create_merge_request(self, git_url, branch_from, title, description, issue_id, to_branch, label):
        return

    @abc.abstractmethod
    def validate_and_close_merge_request(self, git_url, branch):
        pass

    @abc.abstractmethod
    def validate_merge_request_by_url_and_branches(self, git_url, branch_from, branch_to):
        pass

    @abc.abstractmethod
    def find_merge_request_by_commit_message(self, commit_message):
        return



#!/usr/bin/env python
# encoding: utf-8

import os

from gitflow_api.project.angular_project import AngularProject
from gitflow_api.project.maven_project import MavenProject
from gitflow_api.project.python_project import PythonProject


class ProjectManagerStrategy:

    @staticmethod
    def get_instance(path):
        os.chdir(path)
        if os.path.exists('pom.xml'):
            return MavenProject(path)
        elif os.path.exists('package.json'):
            return AngularProject(path)
        elif os.path.exists('setup.py'):
            return PythonProject(path)
        else:
            raise NotImplementedError('Project not supported!')



#!/usr/bin/env python

# Config file
CONFIG_FILE = 'gitflow.config'

# Branches structure
DEFAULT_MASTER_BRANCH = 'master'
DEFAULT_STAGING_BRANCH = 'staging'

DEFAULT_HOTFIX_BRANCH = 'hotfix/{}'
DEFAULT_FEATURE_BRANCH = 'feature/{}'
DEFAULT_RELEASE_BRANCH = 'release/{}'

DEFAULT_VERSION = '{}-{}'

# Issue Classification - Api Labels
DEFAULT_FEATURE_LABELS = ['story', 'feature']
DEFAULT_CHANGE_LABELS = ['change']
DEFAULT_BUG_LABELS = ['bug']
DEFAULT_TECHNICAL_DEBT_LABELS = ['technical debt']
DEFAULT_IGNORE_LABELS = ['ignore']
DEFAULT_VERSION_LABEL_PREFIX = 'v.{major}.{minor}.{patch}'



#!/usr/bin/env python
# encoding: utf-8
from subprocess import check_call

from gitflow_api.deploy.deploy import Deploy


class Maven(Deploy):

    def __init__(self):
        super(Maven, self).__init__(Maven.__class__)
        self.__qualname__ = 'Maven'

    def deploy(self):
        mvn_cmd = 'mvn -B deploy -DskipTests'
        check_call(mvn_cmd, shell=True)



#!/usr/bin/env python
# encoding: utf-8

import abc
import os

from gitflow_api.api.api_merge_request import ApiMergeRequest

class Api(object):
    __metaclass__ = abc.ABCMeta

    global api_url
    global api_key
    global path

    def __init__(self, api_key, api_url):
        self.api_key = api_key
        self.api_url = api_url

        self.path = os.getcwd()

    @abc.abstractmethod
    def get_merge_request_api(self):
        return ApiMergeRequest()



#!/usr/bin/env python
# encoding: utf-8

from gitflow_api.communicator.communicator import CommunicatorType
from gitflow_api.communicator.slack import Slack
from gitflow_api.config.config import Config


class CommunicatorStrategy:

    @staticmethod
    def get_instance():
        config = CommunicatorStrategy._get_config()

        if config.communicator_type == CommunicatorType.SLACK.value:
            return Slack(config.communicator_release_webhook, config.communicator_launch_webhook)
        elif config.communicator_type is None:
            raise ModuleNotFoundError('Communicator not configured!')
        else:
            raise NotImplementedError('Communicator not supported!')

    @staticmethod
    def _get_config():
        return Config()



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase, mock
from unittest.mock import patch, PropertyMock

from gitflow_api.communicator.communicator_strategy import CommunicatorStrategy
from gitflow_api.communicator.slack import Slack
from gitflow_api.config.config import Config
from gitflow_api.services.changelog import Changelog, Issue, ChangelogIssues


class TestCommunicator(TestCase):

    @staticmethod
    def fake_config():
        config = Config()
        config.communicator_type = 'slack'
        config.communicator_launch_webhook = 'http://slack.com/LAUNCH'
        config.communicator_release_webhook = 'http://slack.com/RELEASE'
        return config

    @staticmethod
    def fake_changelog():
        stories = [Issue('Story 1 - Fake screen for fake app', 'http://127.0.0.1', 'story', TestCommunicator.fake_config()),
                   Issue('Story 2 - Fake feature with fake button', 'http://127.0.0.1', 'story', TestCommunicator.fake_config())]
        bugs = [Issue('Bug 1 - Fake bug fixed', 'http://127.0.0.1', 'bug', TestCommunicator.fake_config())]
        changelog = ChangelogIssues()
        changelog.stories = stories
        changelog.bugs = bugs
        changelog.version = '1.0.0'
        return changelog

    @patch.object(Changelog, '_get_config', fake_config)
    def test_make_changelog_slack_format(self):
        slack_message = Slack._make_changelog_slack_format(TestCommunicator.fake_changelog(), 'Fake Project')
        self.assertEqual(slack_message, '*Fake Project - 1.0.0*\n*Improvements*\n- <http://127.0.0.1|Story 1 - Fake screen for fake app>\n- <http://127.0.0.1|Story 2 - Fake feature with fake button>\n*Bugs*\n- <http://127.0.0.1|Bug 1 - Fake bug fixed>')

    @patch.object(CommunicatorStrategy, '_get_config', fake_config)
    def test_get_slack_instance(self):
        slack = CommunicatorStrategy.get_instance()
        self.assertTrue(slack.__class__.__name__ == 'Slack')



#!/usr/bin/env python
# encoding: utf-8
import json

from gitflow_api.project.project_manager import ProjectManager


class AngularProject(ProjectManager):

    def __init__(self, path):
        super().__init__(path)

    def update_dependencies_version(self):
        pass

    def actual_version(self):
        with open(AngularProject._get_version_filename()) as f:
            data = json.load(f)
            return data["version"]

    def update_version(self, version):
        with open(AngularProject._get_version_filename()) as f:
            data = json.load(f)
            data["version"] = version
            return AngularProject._write_new_version(data, AngularProject._get_version_filename())

    def dependencies(self):
        return []

    def test(self):
        return

    def deploy_local(self):
        return
    
    def deploy(self):
        return

    @staticmethod
    def _get_version_filename():
        return 'package.json'

    @staticmethod
    def _write_new_version(data, version_filename):
        version_file = open(version_filename, 'w')
        json.dump(data, version_file, indent=4)
        version_file.truncate()



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase
from unittest.mock import patch
from gitflow_api.utilities.git_helper import GitHelper


class TestGitHelper(TestCase):

    @staticmethod
    def fake_get_api_config_from_config():
        return 'https://git.domain.com.br'

    @patch.object(GitHelper, 'get_api_url_from_config', fake_get_api_config_from_config)
    def test_extract_group_and_project_from_url_ssh(self):
        url = "git@git.domain.com.br:group/project.git"
        group, project = GitHelper.extract_group_and_project_from_url(url)
        self.assertEqual(group, 'group')
        self.assertEqual(project, 'project')

    @patch.object(GitHelper, 'get_api_url_from_config', fake_get_api_config_from_config)
    def test_extract_group_and_project_from_url_http(self):
        url = "https://git.domain.com.br/group/project.git"
        group, project = GitHelper.extract_group_and_project_from_url(url)
        self.assertEqual(group, 'group')
        self.assertEqual(project, 'project')

    @patch.object(GitHelper, 'get_api_url_from_config', fake_get_api_config_from_config)
    def test_extract_group_and_subgroup_and_project_from_url_http(self):
        url = "https://git.domain.com.br/group/subgroup/project.git"
        group, project = GitHelper.extract_group_and_project_from_url(url)
        self.assertEqual(group, 'group/subgroup')
        self.assertEqual(project, 'project')

    @patch.object(GitHelper, 'get_api_url_from_config', fake_get_api_config_from_config)
    def test_extract_group_and_subgroup_and_project_from_url_ssh(self):
        url = "git@git.domain.com.br:group/subgroup/project.git"
        group, project = GitHelper.extract_group_and_project_from_url(url)
        self.assertEqual(group, 'group/subgroup')
        self.assertEqual(project, 'project')



#!/usr/bin/env python
# encoding: utf-8

import os
import re

from gitflow_api.api.gitlab_manager.project import Project
from gitflow_api.api.api_merge_request import ApiMergeRequest


class MergeRequest(ApiMergeRequest):
    global project_api

    def __init__(self, gl):
        super(MergeRequest, self).__init__(gl)
        self.project_api = Project(gl)

    def find_merge_request_by_url_and_branch(self, git_url, branch):
        project = self.project_api.find_project_by_url(git_url)

        return self._find_merge_request_by_project_and_branch(project, branch)

    def create_merge_request(self, git_url, branch_from, title, description, issue_id, to_branch, label):
        project = self.project_api.find_project_by_url(git_url)

        merge_request = self._find_merge_request_by_url_and_branches(git_url, branch_from, to_branch)

        if merge_request is None:
            merge_request_md = self._get_merge_request_md(self.path)

            description = self._make_merge_request_description(merge_request_md, description, issue_id)
            labels = []
            if label is not None:
                labels = [label]

            merge_request_model = {
                'source_branch': branch_from,
                'target_branch': to_branch,
                'title': (branch_from if title is None else title),
                'labels': labels,
                'description': description,
                'remove_source_branch': True
            }

            merge_request = project.mergerequests.create(merge_request_model)

        return merge_request

    def validate_and_close_merge_request(self, git_url, branch):
        mergeRequest = self.find_merge_request_by_url_and_branch(git_url, branch)

        if self._validate_merge_request(mergeRequest):
            mergeRequest.merge()

    def validate_merge_request_by_url_and_branches(self, git_url, branch_from, branch_to):
        merge_request = self._find_merge_request_by_url_and_branches(git_url, branch_from, branch_to)

        return self._validate_merge_request(merge_request)

    def find_merge_request_by_commit_message(self, group_name, project_name, commit_message):
        merge_request_code = commit_message[commit_message.find('See merge request ') + 18:]
        merge_request_id = re.search('([1-9])+', str(merge_request_code[merge_request_code.find('!') + 1:])).group(0)

        group = self.project_api.find_group_by_name(group_name)
        project = self.project_api.find_project_by_group_and_name(group, project_name)
        return project.mergerequests.get(merge_request_id)

    def _find_merge_request_by_project_and_branch(self, project, branch):
        if project.mergerequests is None:
            return None

        merge_requests = project.mergerequests.list(state='opened')

        filtered_merge_requests = list(filter(lambda x: str(x.source_branch) == str(branch), merge_requests))
        if len(filtered_merge_requests) == 0:
            return None
        else:
            return filtered_merge_requests[0]

    def _find_merge_request_by_url_and_branches(self, git_url, branch_from, branch_to):
        project = self.project_api.find_project_by_url(git_url)

        if project.mergerequests is None:
            return None

        merge_requests = project.mergerequests.list(state='opened')

        filtered_merge_requests = list(
            filter(lambda x: str(x.source_branch) == str(branch_from) and str(x.target_branch) == str(branch_to),
                   merge_requests))
        if len(filtered_merge_requests) == 0:
            return None
        else:
            return filtered_merge_requests[0]

    def _make_merge_request_description(self, model, description, issue_id):
        if description is not None:
            return description

        if model is not None:
            replaced_model = model
        else:
            replaced_model = 'Closes #0'

        if issue_id is not None:
            replaced_model.replace('Closes #0', 'Closes #' + str(issue_id))

        return replaced_model

    def _get_merge_request_md(self, path):
        merge_request_md_file = path + '/.gitlab_manager/merge_request_templates/merge_request.md'

        if os.path.exists(merge_request_md_file):
            F = open(merge_request_md_file, 'r')
            return F.read()
        else:
            return None

    def _validate_merge_request(self, merge_request):
        if merge_request is None:
            raise RuntimeWarning('Merge request not found for current branch')

        if bool(merge_request.work_in_progress):
            raise ValueError('Merge Request {} - {} on WIP status'.format(merge_request.iid, merge_request.title))

        if bool(merge_request.discussion_locked):
            raise ValueError(
                'Merge Request {} - {} has discussion unresolved'.format(merge_request.iid, merge_request.title))

        if str(merge_request.merge_status) != 'can_be_merged':
            raise ValueError(
                'Merge Request {} - {} is not ready for merge'.format(merge_request.iid, merge_request.title))

        return True

    def _validate_merge_request_by_url_and_branch(self, git_url, branch):
        merge_request = self.find_merge_request_by_url_and_branch(git_url, branch)
        return self._validate_merge_request(merge_request)

    def _validate_and_close_merge_request_by_branches(self, git_url, branch_from, branch_to):
        merge_request = self._find_merge_request_by_url_and_branches(git_url, branch_from, branch_to)

        if self._validate_merge_request(merge_request):
            merge_request.merge()



#!/usr/bin/env python
# encoding: utf-8

import gitlab

from gitflow_api.api.api import Api
from gitflow_api.api.gitlab_manager.merge_request import MergeRequest


class GitlabManager(Api):

    def get_merge_request_api(self):
        return MergeRequest(self.gl)

    global gl

    def __init__(self, api_key, api_url):
        super(GitlabManager, self).__init__(api_key, api_url)

        self.gl = gitlab.Gitlab(self.api_url, private_token=self.api_key)



#!/usr/bin/env python
# encoding: utf-8
import json
import os

from gitflow_api.api.api_strategy import ApiStrategy
from gitflow_api.communicator.communicator_strategy import CommunicatorStrategy
from gitflow_api.config.config import Config
from gitflow_api.utilities.git_helper import GitHelper
from gitflow_api.project.project_manager_strategy import ProjectManagerStrategy
from gitflow_api.utilities.version_utils import VersionUtils, Version

STORY_TYPE = 'story'
BUG_TYPE = 'bug'
TECHNICAL_TYPE = 'technical_debt'
OTHER_TYPE = 'other'
IGNORE_TYPE = 'ignore'


class Changelog:
    config = None

    def __init__(self):
        pass

    def create_changelog(self, branch, from_tag=None, path=None, only_staging=False):
        git = GitHelper()
        branch = str(git.get_current_branch() if branch is None else branch)

        return self._create_changelog(branch, from_tag=from_tag, path=path, only_staging=only_staging)

    def create_markdown_changelog(self, branch, from_tag=None, path=None, only_staging=False, write_changelog=False, send=False):
        changelog_issues = self.create_changelog(branch, from_tag=from_tag, path=path, only_staging=only_staging)

        changelog_md = self.make_changelog_md(changelog_issues)

        if write_changelog:
            self.write_changelog(changelog_issues)

        if send:
            self._post_changelog(changelog_issues)

        return changelog_md

    def write_changelog(self, changelog_issues):
        if bool(self._get_config().changelog_create_file):
            git = GitHelper()
            group_name, project_name = GitHelper.extract_group_and_project_from_url(git.get_current_url())

            filename = self._get_config().version.format(project_name, version)

            path = self._get_config().changelog_path
            if not os.path.exists(path):
                os.makedirs(path)

            changelog_json = json.dumps(changelog_issues, sort_keys=True, indent=4, cls=ComplexEncoder, ensure_ascii=False)

            release_notes = open(path + './' + filename, 'w', encoding='utf8')
            release_notes.write(changelog_json)
            release_notes.close()

    def _create_changelog(self, branch, from_tag=None, path=None, only_staging=False):

        if path is not None:
            os.chdir(path)

        git = GitHelper()
        git.checkout_and_pull(branch)

        if len(git.repo.tags) == 0:
            return ChangelogIssues()

        tag_commit = ''
        if from_tag is not None:
            tag_commit = git.repo.tags[from_tag].commit
        else:
            tags = sorted(git.repo.tags, key=lambda x: x.commit.authored_date, reverse=True)

            group_name, project_name = git.extract_group_and_project()
            tag_commit = self._find_last_tag(project_name, tags)

        print('Creating changelog from {}'.format(str(tag_commit)))
        log = git.get_git_cmd().log(str(tag_commit) + '..HEAD').split('\n')

        commits = self._get_commit_by_log(log)

        group_name, project_name = GitHelper.extract_group_and_project_from_url(git.get_current_url())

        project = ProjectManagerStrategy().get_instance(os.getcwd())
        version = project.actual_version()

        changelog_issues = self._normalize_issues(group_name, project_name, commits, only_staging)
        changelog_issues.version = version
        return changelog_issues

    def _find_last_tag(self, project_name, tags):
        path = os.getcwd()

        project_management = ProjectManagerStrategy.get_instance(path)
        actual_version = project_management.actual_version().replace('-SNAPSHOT', '')
        # ultima tag
        tag_commit = tags[0]
        tag_version = str(tag_commit).replace(self._get_config().version.format(project_name, ''), '')

        if VersionUtils.extract_version(tag_version, Version.MAJOR) != VersionUtils.extract_version(
                actual_version, Version.MAJOR):
            last_version = '{}.0.0'.format(VersionUtils.extract_version(actual_version, Version.MAJOR) - 1)
            for tag in tags:
                tag_commit = tag
                tag_version = str(tag_commit).replace(self._get_config().version.format(project_name, ''), '')

                if VersionUtils.diff_version(tag_version, last_version) <= 0:
                    break
        else:

            if VersionUtils.extract_version(tag_version, Version.MINOR) != VersionUtils.extract_version(
                    actual_version, Version.MINOR):
                last_version = '{}.{}.0'.format(VersionUtils.extract_version(actual_version, Version.MAJOR),
                                                VersionUtils.extract_version(actual_version, Version.MINOR) - 1)
                for tag in tags:
                    tag_commit = tag
                    tag_version = str(tag_commit).replace(self._get_config().version.format(project_name, ''), '')

                    if VersionUtils.diff_version(tag_version, last_version) <= 0:
                        break

        return tag_commit

    def _normalize_issues(self, group_name, project_name, commits, only_staging):
        api = ApiStrategy.get_instance(os.getcwd())
        changelog_issues = ChangelogIssues()

        for commit in commits:
            # print(commit.commit)
            # print(commit.message)

            message = commit.message
            if message.find('See merge request') >= 0:

                merge_request = api.get_merge_request_api().find_merge_request_by_commit_message(group_name,
                                                                                                 project_name, message)

                add_changelog = merge_request.source_branch.find(self._get_config().release_branch.format('')) == -1
                add_changelog = add_changelog and (
                        merge_request.target_branch == self._get_config().staging_branch or not only_staging)
                add_changelog = add_changelog and merge_request.state == 'merged'
                if add_changelog:
                    issue = Issue(merge_request.title, merge_request.web_url,
                                  merge_request.labels, self._get_config())
                    self._add_change_log_issue(changelog_issues, issue)

            # ignore some commits
            elif message.find('[maven-release-plugin]') >= 0 or message.find('Merge branch') >= 0:
                pass

        return changelog_issues

    def _add_change_log_issue(self, changelog_issues, issue):
        if issue.issueType == IGNORE_TYPE:
            pass
        elif issue.issueType == STORY_TYPE:
            changelog_issues.stories.append(issue)
        elif issue.issueType == BUG_TYPE:
            changelog_issues.bugs.append(issue)
        elif issue.issueType == TECHNICAL_TYPE:
            changelog_issues.technicalDebts.append(issue)
        else:
            changelog_issues.others.append(issue)

    def _get_commit_by_log(self, log):
        commits = []
        actual_commit = Commit()
        for logLine in log:
            line = str(logLine)
            if line.find('commit') >= 0:
                if actual_commit.commit != '':
                    commits.append(actual_commit)
                actual_commit = Commit()
                actual_commit.commit = line.replace('commit ', '')
            elif line.find('Author:') >= 0:
                actual_commit.author = line.replace('Author: ', '')
            elif line.find('Date:') >= 0:
                actual_commit.date = line.replace('Date:   ', '')
            else:
                actual_commit.message = actual_commit.message + ('' if actual_commit.message == '' else '\n') + str(
                    line)

        return commits

    @staticmethod
    def make_changelog_md(changelog_issues):
        merge_request_md = ''
        if len(changelog_issues.stories) > 0:
            merge_request_md = merge_request_md + '\n' + str('## Improvements')
            for issue in changelog_issues.stories:
                merge_request_md = merge_request_md + '\n' + str('* [{}]({})').format(issue.title, issue.url)

        if len(changelog_issues.bugs) > 0:
            merge_request_md = merge_request_md + '\n' + str('## Bugs')
            for issue in changelog_issues.bugs:
                merge_request_md = merge_request_md + '\n' + str('* [{}]({})').format(issue.title, issue.url)

        if len(changelog_issues.technicalDebts) > 0:
            merge_request_md = merge_request_md + '\n' + str('## Technical Debts')
            for issue in changelog_issues.technicalDebts:
                merge_request_md = merge_request_md + '\n' + str('* [{}]({})').format(issue.title, issue.url)

        if len(changelog_issues.others) > 0:
            merge_request_md = merge_request_md + '\n' + str('## Others')
            for issue in changelog_issues.others:
                merge_request_md = merge_request_md + '\n' + str('* [{}]({})').format(issue.title, issue.url)

        return merge_request_md

    def _get_config(self):
        if (self.config is None):
            self.config = Config()

        return self.config

    def _post_changelog(self, changelog):
        git = GitHelper()
        group_name, project_name = GitHelper.extract_group_and_project_from_url(git.get_current_url())

        communicator = CommunicatorStrategy.get_instance()
        communicator.send_changelog(changelog, communicator.release_webhook, project_name)

class ChangelogIssues:
    version = '0.0.0'
    stories = []
    bugs = []
    technicalDebts = []
    others = []

    def __init__(self):
        pass

    def toJSON(self):
        return dict(version=self.version, stories=self.stories, bugs=self.bugs, technicalDebts=self.technicalDebts, others=self.others)


class Issue:
    title = ''
    url = ''
    issueType = ''

    def __init__(self, title, url, labels, config):
        self.title = str(title)
        self.url = str(url)
        self.issueType = self._getIssueType(labels, config)

    def _getIssueType(self, labels, config):
        if len(list(filter(lambda x: str(x).find(config.version_label_prefix.format('')) == 0,
                           labels))) > 0 or len(set(config.ignore_labels) & set(labels)) > 0:
            return IGNORE_TYPE
        elif len(set(config.feature_labels) & set(labels)) > 0:
            return STORY_TYPE
        elif len(set(config.bug_labels) & set(labels)) > 0:
            return BUG_TYPE
        elif len(set(config.technical_debt_labels) & set(labels)) > 0:
            return TECHNICAL_TYPE
        else:
            return OTHER_TYPE

    def toJSON(self):
        return dict(title=self.title, url=self.url, issueType=self.issueType)


class Commit:
    commit = ''
    author = ''
    date = ''
    message = ''

    def __init__(self):
        pass


class ComplexEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, 'toJSON'):
            return obj.toJSON()
        else:
            return json.JSONEncoder.default(self, obj)



#!/usr/bin/env python
# encoding: utf-8
from gitflow_api.config.config import Config
from gitflow_api.deploy.deploy import Deploy


class DeployStrategy:

    @staticmethod
    def get_instance(default_deploy_class):
        config = DeployStrategy._get_config()
        if config.extension_deploy_class is not None:
            package, classname = DeployStrategy.extract_package_and_classname(config.extension_deploy_class)
        else:
            package, classname = DeployStrategy.extract_package_and_classname(default_deploy_class)

        exec('from {} import {}'.format(package, classname))
        return eval('{}()'.format(classname))

    @staticmethod
    def extract_package_and_classname(fullname):
        last_dot = fullname.rfind('.')
        return fullname[:last_dot], fullname[last_dot+1:]

    @staticmethod
    def _get_config():
        return Config()


#!/usr/bin/env python
# encoding: utf-8
import json
import requests

from gitflow_api.communicator.communicator import Communicator


class Slack(Communicator):

    def __init__(self, release_webhook, launch_webhook):
        super(Slack, self).__init__(release_webhook, launch_webhook)

    def send_message(self, message, channel, project_name):
        json = SlackPostMessage(message, 'Gitflow-API').toJSON()
        r = requests.post(channel, json=json)
        return requests.post(channel, json=json)

    def send_changelog(self, changelog, channel, project_name):
        message = SlackPostMessage(self._make_changelog_slack_format(changelog, project_name), 'Gitflow-API').toJSON()
        return requests.post(channel, data=message)

    @staticmethod
    def _make_changelog_slack_format(changelog_issues, project_name):
        slack_format_changelog = []
        slack_format_changelog.append('*{} - {}*'.format(project_name, changelog_issues.version))
        if len(changelog_issues.stories) > 0:
            slack_format_changelog.append(str('*Improvements*'))
            for issue in changelog_issues.stories:
                slack_format_changelog.append(str('- <{}|{}>').format(issue.url, issue.title))

        if len(changelog_issues.bugs) > 0:
            slack_format_changelog.append(str('*Bugs*'))
            for issue in changelog_issues.bugs:
                slack_format_changelog.append(str('- <{}|{}>').format(issue.url, issue.title))

        if len(changelog_issues.technicalDebts) > 0:
            slack_format_changelog.append(str('*Technical Debts*'))
            for issue in changelog_issues.technicalDebts:
                slack_format_changelog.append(str('- <{}|{}>').format(issue.url, issue.title))

        if len(changelog_issues.others) > 0:
            slack_format_changelog.append(str('*Others*'))
            for issue in changelog_issues.others:
                slack_format_changelog.append(str('- <{}|{}>').format(issue.url, issue.title))

        return '\n'.join(slack_format_changelog)


class SlackPostMessage:
    text = ''
    username = ''
    mrkdwn = True

    def __init__(self, text, username):
        self.text = text
        self.username = username
        self.mrkdwn = True

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


#!/usr/bin/env python
# encoding: utf-8

import os
from enum import Enum

from gitflow_api.api.gitlab_manager.gitlab_manager import GitlabManager
from gitflow_api.config.config import Config


class ApiStrategy:
    API_NOT_SUPPORTED = 'Api not supported!'

    @staticmethod
    def get_instance(path):
        os.chdir(path)
        config = Config()

        if config.api_type == ApiType.GITLAB.value:
            return GitlabManager(config.api_key, config.api_url)
        elif config.api_type == ApiType.GITHUB.value:
            return None

        raise NotImplementedError(ApiStrategy.API_NOT_SUPPORTED)


class ApiType(Enum):
    GITLAB = "gitlab"
    GITHUB = "github"



#!/usr/bin/env python
# encoding: utf-8
import argparse

from gitflow_api.services.feature import Feature
from gitflow_api.services.hotfix import Hotfix
from gitflow_api.services.release import Release
from gitflow_api.services.changelog import Changelog
from gitflow_api.utilities.version_utils import VersionUtils


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('action', type=str,
                        help='Actions: feature-start, feature-finish, hotfix-start, hotfix-finish, release-start, '
                             'release-finish, launch, changelog, adjust-versions')
    parser.add_argument('branch', type=str, nargs='?', const=None, help='Branch name')
    parser.add_argument('--title', help='Title description of feature or hotfix')
    parser.add_argument('--issue', help='Issue ID on GitLab')
    parser.add_argument('--fromTag', help='Initial tag for changelog comparing')
    parser.add_argument('--onlyStaging', nargs='?', const=True, help='Use only staging merge requests for changelog')
    parser.add_argument('--force', nargs='?', const=True, help='Force to recreate release')
    parser.add_argument('--skipTests', nargs='?', const=True, help='Skip tests')
    parser.add_argument('--writeChangelog', nargs='?', const=True, help='Force write changelog file')
    parser.add_argument('--send', nargs='?', const=True, help='Send changelog to communicator')

    args = parser.parse_args()

    if args.action.lower() == 'feature-start':
        Feature().feature_start(args)
    elif args.action.lower() == 'feature-finish':
        Feature().feature_finish(args)
    elif args.action.lower() == 'hotfix-start':
        Hotfix().hotfix_start(args)
    elif args.action.lower() == 'hotfix-finish':
        Hotfix().hotfix_finish(args)
    elif args.action.lower() == 'release-start':
        Release().release_start(force=args.force, skip_tests=args.skipTests)
    elif args.action.lower() == 'release-finish':
        Release().release_finish(args)
    elif args.action.lower() == 'launch':
        Release().launch(force=args.force)
    elif args.action.lower() == 'changelog':
        print(Changelog().create_markdown_changelog(args.branch, from_tag=args.fromTag, only_staging=args.onlyStaging,
                                                    write_changelog=args.writeChangelog, send=args.send))
    elif args.action.lower() == 'adjust-versions':
        VersionUtils.adjust_versions()
    else:
        print(
            'Action not found [feature-start, feature-finish, hotfix-start, hotfix-finish, release-start, '
            'release-finish, launch, changelog, adjust-versions]')


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8

import abc
import os
from enum import Enum

from gitflow_api.api.api_merge_request import ApiMergeRequest


class Communicator(object):
    __metaclass__ = abc.ABCMeta

    release_webhook = ''
    launch_webhook = ''

    def __init__(self, release_webhook, launch_webhook):
        self.release_webhook = release_webhook
        self.launch_webhook = launch_webhook

    @abc.abstractmethod
    def send_message(self, message, channel, project_name):
        pass

    @abc.abstractmethod
    def send_changelog(self, changelog, channel, project_name):
        pass


class CommunicatorType(Enum):
    SLACK = "slack"



#!/usr/bin/env python
from setuptools import setup, find_packages

install_requires = ['GitPython>=2.1.11','python-gitlab>=1.6.0','slackclient>=1.3.0']
test_requires = []
# end

setup(
    name="gitflow-api",
    version="0.3.8",
    description="Gitflow-API library using a API as backend",
    author="Mauyr Alexandre Pereira",
    author_email="mauyr.pereira@inovapro.com.br",
    url="https://github.com/mauyr/gitflow-api",
    packages=find_packages('.'),
    entry_points = {
        'console_scripts': ['gitflow=gitflow_api.gitflow:main'],
    },
    license="GNU GPLv3",
    python_requires='>=3.0',
    install_requires=install_requires,
    long_description="Gitflow-API is a python library used use a gitflow_methodology on development using a API like "
                     "Gitlab or Github to execute most of process",
    classifiers=[
        # Picked from
        #   http://pypi.python.org/pypi?:action=list_classifiers
        # "Development Status :: 1 - Planning",
        "Development Status :: 2 - Pre-Alpha",
        # "Development Status :: 3 - Alpha",
        # "Development Status :: 4 - Beta",
        # "Development Status :: 5 - Production/Stable",
        # "Development Status :: 6 - Mature",
        # "Development Status :: 7 - Inactive",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS :: MacOS X",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7"
    ]
)


#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase
from unittest.mock import patch

from gitflow_api.project.python_project import PythonProject
import os


class TestPythonProject(TestCase):

    @staticmethod
    def get_correct_setup_file():
        return 'project/mock/setup.py.mock'

    @patch.object(PythonProject, '_get_version_filename', get_correct_setup_file)
    def test_actual_version_with_correct_setup(self):
        python_project = PythonProject(os.getcwd())
        actual_version = python_project.actual_version()
        self.assertEqual(actual_version, '1.0.0')

    @staticmethod
    def get_incorrect_version_setup_file():
        return 'project/mock/incorrect_version_setup.py.mock'

    @patch.object(PythonProject, '_get_version_filename', get_incorrect_version_setup_file)
    def test_actual_version_with_incorrect_version_setup(self):
        with self.assertRaises(ValueError) as context:
            python_project = PythonProject(os.getcwd())
            python_project.actual_version()

        self.assertTrue('Version has incorrect format' in context.exception.args)

    @staticmethod
    def get_incorrect_setup_file():
        return 'project/mock/incorrect_setup.py.mock'

    @patch.object(PythonProject, '_get_version_filename', get_incorrect_setup_file)
    def test_actual_version_with_incorrect_setup(self):
        with self.assertRaises(ValueError) as context:
            python_project = PythonProject(os.getcwd())
            python_project.actual_version()

        self.assertTrue('Version tag not found' in context.exception.args)

    @staticmethod
    def write_fake_setup(lines, version_file):
        pass

    @patch.object(PythonProject, '_get_version_filename', get_correct_setup_file)
    @patch.object(PythonProject, '_write_new_version', write_fake_setup)
    def test_update_version(self):
        python_project = PythonProject(os.path)
        new_file = python_project.update_version('2.0.0')

        self.assertTrue(len(list(filter(lambda x: str(x).find('version="2.0.0"') >= 0, new_file))) > 0)

        print(new_file)







#!/usr/bin/env python
# encoding: utf-8
import configparser
import os
from subprocess import check_call, check_output, call

from git import Repo

from gitflow_api.config.config import Config
from gitflow_api.config.properties import CONFIG_FILE


class GitHelper:
    global repo
    global path

    config = Config()

    def __init__(self):
        self.data = []

        self.path = os.getcwd()
        self.repo = Repo(self.path)

    def get_current_url(self):
        return self.repo.remotes[0].url

    def get_current_branch(self):
        return self.repo.active_branch

    def create_new_branch_from(self, from_branch, new_branch):
        self.checkout_and_pull(from_branch)

        git_cmd = 'git checkout -b {}'
        check_call(git_cmd.format(new_branch), shell=True)

        self.push_branch(new_branch)

    def checkout_and_pull(self, branch):
        try:
            git_cmd = 'git checkout {}'
            check_call(git_cmd.format(branch), shell=True)

            git_cmd = 'git pull'
            check_call(git_cmd.format(branch), shell=True)
            return True
        except Exception as e:
            return False

    def delete_branch(self, branch):
        try:
            git_cmd = 'git branch -D {}'
            check_call(git_cmd.format(branch), shell=True)
        except Exception as e:
            pass

    def commit_and_push_update_message(self, branch, version):
        self.checkout_and_pull(branch)

        try:
            git_cmd = 'git add .'
            check_call(git_cmd, shell=True)

            git_cmd = 'git commit -m "Update to next version {}"'
            call(git_cmd.format(version), shell=True)
        except Exception as e:
            print('Nothing to commmit. Skipping.')
            return

        self.push_branch(branch)

    def push_branch(self, branch):
        git_cmd = 'git push -u origin {}'
        check_call(git_cmd.format(branch), shell=True)

    def create_tag(self, tag, message):
        git_cmd = 'git tag -a "{}" -m "{}"'
        check_call(git_cmd.format(tag, message), shell=True)

        self.push_branch(tag)

    def delete_tag(self, tag):
        git_cmd = 'git push --delete origin {}'
        check_call(git_cmd.format(tag), shell=True)

        git_cmd = 'git tag -d {}'
        check_call(git_cmd.format(tag), shell=True)

    def check_conflicts(self, from_branch, to_branch):
        try:
            git_cmd = 'git merge --no-commit {}'
            check_call(git_cmd.format(from_branch), shell=True)

            git_cmd = 'git commit -m "{}"'
            call(git_cmd.format('Automatic merge maked by gitflow-api'), shell=True)

            self.push_branch(to_branch)
        except Exception:
            git_cmd = 'git reset HEAD --hard'
            check_call(git_cmd, shell=True)

            group_name, project_name = self.extract_group_and_project()

            raise RuntimeError('Project {} has conflict from branch {} into {}'.format(project_name, from_branch, to_branch))

    def check_branch_exists_on_remote(self, branch):
        git_cmd = 'git ls-remote --heads origin {} | wc -l'
        return int(check_output(git_cmd.format(branch), shell=True).decode('UTF-8'))

    def extract_group_and_project(self):
        return GitHelper.extract_group_and_project_from_url(self.get_current_url())

    @staticmethod
    def extract_group_and_project_from_url(git_url):
        api_url = GitHelper.get_api_url_from_config()

        short_api_url = GitHelper.extract_url_without_protocol(api_url)
        short_git_url = GitHelper.extract_url_without_protocol(git_url)

        only_groups_and_project = short_git_url.replace(short_api_url, '')[1:].split('/')
        project = str(only_groups_and_project[len(only_groups_and_project)-1]).replace('.git', '')
        del only_groups_and_project[len(only_groups_and_project)-1]
        group = '/'.join(only_groups_and_project)
        return group, project

    @staticmethod
    def get_api_url_from_config():
        config = Config()
        return config.api_url

    def get_git_cmd(self):
        return self.repo.git

    @staticmethod
    def extract_url_without_protocol(url):
        initial_pos = url.find('://')
        initial_pos = url.find('@')+1 if initial_pos == -1 else initial_pos+3
        return url[initial_pos:]



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase, mock
from unittest.mock import patch

from gitflow_api.config.config import Config
from gitflow_api.deploy.deploy_strategy import DeployStrategy


class TestDeploy(TestCase):

    @staticmethod
    def fake_config():
        config = Config()
        config.extension_deploy_class = 'gitflow_api.deploy.maven.Maven'
        return config

    @staticmethod
    def fake_config_without_deploy_class():
        config = Config()
        return config

    @patch.object(DeployStrategy, '_get_config', fake_config)
    def test_get_configured_instance(self):
        instance = DeployStrategy.get_instance('gitflow_api.deploy.pypi.Pypi')
        self.assertTrue(instance.__qualname__ == 'Maven')

    @patch.object(DeployStrategy, '_get_config', fake_config_without_deploy_class)
    def test_get_pypi_instance(self):
        instance = DeployStrategy.get_instance('gitflow_api.deploy.pypi.Pypi')
        self.assertTrue(instance.__qualname__ == 'Pypi')



#!/usr/bin/env python
# encoding: utf-8

from unittest import TestCase

from gitflow_api.utilities.version_utils import VersionUtils, Version


class TestVersionUtils(TestCase):
    def test_get_new_version_patch(self):
        new_version = VersionUtils.get_new_version('1.0.0', Version.PATCH, False, 1)
        self.assertEqual(new_version, '1.0.1')

    def test_get_new_version_minor(self):
        new_version = VersionUtils.get_new_version('1.0.0', Version.MINOR, False, 1)
        self.assertEqual(new_version, '1.1.0')

    def test_get_new_version_major(self):
        new_version = VersionUtils.get_new_version('1.0.0', Version.MAJOR, False, 1)
        self.assertEqual(new_version, '2.0.0')

    def test_get_new_version_patch_snapshot(self):
        new_version = VersionUtils.get_new_version('1.0.0', Version.PATCH, True, 1)
        self.assertEqual(new_version, '1.0.1-SNAPSHOT')

    def test_get_new_version_patch_decrement_invalid(self):
        new_version = VersionUtils.get_new_version('1.0.0', Version.PATCH, False, -1)
        self.assertEqual(new_version, '1.0.0')

    def test_get_new_version_patch_decrement_valid(self):
        new_version = VersionUtils.get_new_version('1.0.2', Version.PATCH, False, -1)
        self.assertEqual(new_version, '1.0.1')

    def test_format_version_full_attributes(self):
        version = '1.0.2'
        project_name = 'test'
        format = 'v.{major}.{minor}.{patch}-{project}'
        formatted_version = VersionUtils.format_version(format, version, project_name)
        self.assertEqual(formatted_version, 'v.1.0.2-test')



#!/usr/bin/env python
# encoding: utf-8
import os

from gitflow_api.api.api_strategy import ApiStrategy
from gitflow_api.communicator.communicator_strategy import CommunicatorStrategy
from gitflow_api.config.config import Config
from gitflow_api.utilities.git_helper import GitHelper
from gitflow_api.services.changelog import Changelog
from gitflow_api.utilities.version_utils import VersionUtils, Version
from gitflow_api.project.project_manager_strategy import ProjectManagerStrategy


class Release:
    config = None

    def __init__(self):
        pass

    def release_start(self, force=False, skip_tests=False):
        git = GitHelper()

        group_name, url_pattern = self._url_pattern_build(git)

        initial_path = os.getcwd()

        git = GitHelper()
        git.get_git_cmd().checkout(self._get_config().staging_branch)
        git.get_git_cmd().pull()

        git.check_conflicts(self._get_config().master_branch, self._get_config().staging_branch)

        # print(urlPattern)
        release_branch = self._get_config().release_branch.format(
            self._recursive_release(initial_path, url_pattern, group_name, force, skip_tests))

        self._create_merge_requests(initial_path, release_branch)

        # Get back to initial directory
        os.chdir(initial_path)

    def release_finish(self, args):
        git = GitHelper()
        path = os.getcwd()

        group_name, url_pattern = self._url_pattern_build(git)

        branch = str(git.get_current_branch() if args.branch is None else args.branch)

        self._recursive_finish_release(path, branch, url_pattern, group_name)

    def launch(self, force=False):
        git = GitHelper()
        git.checkout_and_pull(self._get_config().master_branch)

        path = os.getcwd()
        project_management = ProjectManagerStrategy.get_instance(path)

        version = project_management.actual_version().replace('-SNAPSHOT', '')
        group_name, project_name = git.extract_group_and_project()
        formatted_version = VersionUtils.format_version(self._get_config().version, version, project_name)

        if force:
            try:
                git.delete_tag(formatted_version)
            except Exception as e:
                pass

        # TODO: Criar rotinas para release recursivos e verificar se todas as dependencias nao estao mais em snapshot
        project_management.update_version(version)
        project_management.update_dependencies_version()

        git.commit_and_push_update_message(self._get_config().master_branch, formatted_version)

        try:
            changelog_issues = self._create_and_write_changelog()
            self._post_changelog(changelog_issues, 'launch', project_name)
        except Exception as e:
            print("Fail to create changelog", e)

        git.create_tag(formatted_version, version)

        try:
            project_management.deploy()
        except Exception as e:
            print("Fail to deploy project", e)

        # update patch version for next tag
        new_version = self._update_version(Version.PATCH, False)

        git.commit_and_push_update_message(self._get_config().master_branch, new_version)

    def _create_and_write_changelog(self):
        changelog = Changelog()
        changelog_issues = Changelog().create_changelog(self._get_config().master_branch)

        changelog.write_changelog(changelog_issues)

        return changelog_issues

    def _post_changelog(self, changelog, channel, project_name):
        communicator = CommunicatorStrategy.get_instance()
        if channel == 'release':
            communicator.send_changelog(changelog, communicator.release_webhook, project_name)
        else:
            communicator.send_changelog(changelog, communicator.launch_webhook, project_name)

    def _update_version(self, version_update, is_snapshot):
        path = os.getcwd()
        project_management = ProjectManagerStrategy.get_instance(path)
        version = project_management.actual_version().replace('-SNAPSHOT', '')

        new_version = VersionUtils.get_new_version(version, version_update, is_snapshot, +1)

        project_management.update_version(new_version)

        return new_version

    def _recursive_release(self, actual_path, url_pattern, origin_group_name, force, skip_tests):

        project_management = ProjectManagerStrategy.get_instance(actual_path)

        os.chdir(actual_path)

        git = GitHelper()
        git.get_git_cmd().checkout(self._get_config().staging_branch)
        git.get_git_cmd().pull()

        dependencies = project_management.dependencies()
        print('Total dependencies of {}: {}'.format(actual_path, len(dependencies)))

        if len(dependencies) > 0:
            for dependency in dependencies:
                path = self._checkout_dependency(self._get_config().staging_branch, actual_path, dependency, origin_group_name,
                                                 url_pattern)
                try:
                    dependency_project_manager = ProjectManagerStrategy.get_instance(path)
                    os.chdir(path)
                    git = GitHelper()

                    dependency_version = dependency.version.replace('-SNAPSHOT', '')
                    git.get_git_cmd().checkout('release/' + dependency_version)

                    dependency_project_manager.deploy()
                except Exception as e:
                    self._recursive_release(path, url_pattern, origin_group_name, force, skip_tests)

        try:
            os.chdir(actual_path)

            git = GitHelper()
            release_version = project_management.actual_version().replace('-SNAPSHOT', '')

            print('Starting release of {}'.format(os.getcwd()))

            if not skip_tests:
                project_management.test()

            release_branch = self._get_config().release_branch.format(release_version)
            if force:
                git.delete_branch(release_branch)

            if not git.checkout_and_pull(release_branch):
                git.create_new_branch_from(self._get_config().staging_branch, release_branch)

            project_management.deploy_local()

            return release_version
        except RuntimeError as e:
            print('Error creating release of {}: {}'.format(os.getcwd(), str(e)))

    def _recursive_finish_release(self, actual_path, branch, url_pattern, origin_group_name):
        project_management = ProjectManagerStrategy.get_instance(actual_path)
        os.chdir(actual_path)

        dependencies = project_management.dependencies()
        print('Total dependencies of {}: {}'.format(actual_path, len(dependencies)))

        if len(dependencies) > 0:
            for dependency in dependencies:
                path = self._checkout_dependency(self._get_config().staging_branch, actual_path, dependency, origin_group_name,
                                                 url_pattern)

                dependency_version = dependency.version.replace('-SNAPSHOT', '')
                release_branch = 'release/' + dependency_version
                self._recursive_finish_release(path, release_branch, dependency, origin_group_name)

        try:
            os.chdir(actual_path)

            git = GitHelper()
            api = ApiStrategy.get_instance(actual_path)

            if branch.lower().find('release') != 0 or git.check_branch_exists_on_remote(branch) == 0:
                print('Not valid release branch {}. Continuing...'.format(branch))
                return

            print('Finishing release for project {}'.format(git.get_current_url()))

            # continue if dont have conflicts or merge request for release is merged
            try:
                api.get_merge_request_api().validate_and_close_merge_request(git.get_current_url(), branch)
            except RuntimeWarning as ex:
                print('Release branch merged!', ex)
            except ValueError as e:
                raise ValueError(e)

            # Synchronization merge request from master to staging to execute after finish release
            git.get_git_cmd().checkout(self._get_config().master_branch)
            git.get_git_cmd().pull()

            git.get_git_cmd().checkout(self._get_config().staging_branch)
            git.get_git_cmd().pull()
            try:
                git.check_conflicts(self._get_config().master_branch, self._get_config().staging_branch)
            except Exception as e:
                print('Found a conflict on merge master to staging. Trying a automatic merge request: ')
                api.get_merge_request_api().create_merge_request(git.get_current_url(), self._get_config().master_branch,
                                                                 'Synchronization merge request {} to {}'.format(
                                                                     self._get_config().master_branch,
                                                                     self._get_config().staging_branch),
                                                                 'Don\'t remove this merge request before finish release',
                                                                 None,
                                                                 self._get_config().staging_branch, None)
                api.get_merge_request_api().validate_merge_request_by_url_and_branches(git.get_current_url(),
                                                                                       self._get_config().master_branch,
                                                                                       self._get_config().staging_branch)

            git.get_git_cmd().pull()

            new_version = self._update_version(Version.MINOR, True)
            git.commit_and_push_update_message(self._get_config().staging_branch, new_version)

        except RuntimeError as e:
            print('Error finishing release of {}: {}'.format(os.getcwd(), str(e)))

    def _create_merge_requests(self, path, branch):
        os.chdir(path)

        git = GitHelper()
        group_name, url_pattern = self._url_pattern_build(git)

        merge_requests = self._create_recursive_merge_request(branch, path, url_pattern, group_name)

        if merge_requests is not None:
            for mergeRequest in merge_requests:
                print('Merge request created: {}'.format(mergeRequest.iid))

    def _create_recursive_merge_request(self, branch, actual_path, url_pattern, origin_group_name):
        project_management = ProjectManagerStrategy.get_instance(actual_path)

        os.chdir(actual_path)
        git = GitHelper()
        merge_requests = []

        git.get_git_cmd().checkout(branch)
        git.get_git_cmd().pull()

        dependencies = project_management.dependencies()
        print('Total dependencies of {}: {}'.format(actual_path, len(dependencies)))

        if len(dependencies) > 0:
            for dependency in dependencies:
                release_version = dependency.version.replace('-SNAPSHOT', '')
                path = self._checkout_dependency('release/' + release_version, actual_path, dependency,
                                                 origin_group_name,
                                                 url_pattern)

                merge_requests.extend(
                    self._create_recursive_merge_request('release/' + release_version, path, url_pattern,
                                                         origin_group_name))
        api = ApiStrategy.get_instance(actual_path)

        merge_request = api.get_merge_request_api().find_merge_request_by_url_and_branch(git.get_current_url(), branch)
        if merge_request is None:
            description = str('## Merge requests\n')
            for createdMergeRequests in merge_requests:
                description = description + '* ' + createdMergeRequests.web_url + '\n'

            group_name, project_name = git.extract_group_and_project_from_url(git.get_current_url())

            changelog = Changelog().create_changelog(branch, path=actual_path, only_staging=True)
            try:
                self._post_changelog(changelog, 'release', project_name)
            except Exception:
                pass

            description = description + Changelog.make_changelog_md(changelog)

            merge_requests.append(
                api.get_merge_request_api().create_merge_request(git.get_current_url(), branch,
                                                                 'Release {} - ' + branch,
                                                                 description, None, self._get_config().master_branch, None))

        return merge_requests

    def _checkout_dependency(self, branch, actual_path, dependency, origin_group_name, url_pattern):
        print('Checkout from {} dependency {}'.format(actual_path, dependency.artifactId))

        group = origin_group_name
        shared_projects = list(filter(lambda x: dependency.artifactId == x.project, self._get_shared_projects()))
        if len(shared_projects) > 0:
            group = shared_projects[0].group
        shared_projects = list(filter(lambda x: dependency.artifactId == x.project, self._get_shared_projects()))
        if len(shared_projects) > 0:
            group = shared_projects[0].group

        path = '../{}'.format(dependency.artifactId)
        if not os.path.exists(path):
            print('Cloning repository: {}'.format(url_pattern.format(group, dependency.artifactId)))
            new_repo = GitHelper()
            new_repo.repo.clone_from(url_pattern.format(group, dependency.artifactId), path)

            os.chdir(path)
            git_helper = GitHelper()
            git_helper.get_git_cmd().checkout(branch)
            git_helper.get_git_cmd().pull()

        return path

    # List of shared projects
    def _get_shared_projects(self):
        projects = [Project('rf-dominio', 'desenvolvimento'), Project('rf-adapters', 'desenvolvimento'),
                    Project('rf-canonico-aereo', 'gateway'), Project('rf-client-sabre', 'gateway')]
        return projects

    def _url_pattern_build(self, git_helper):
        url = git_helper.get_current_url().split('/')
        group_name = str(url[0][url[0].rfind(':') + 1:])
        url[0] = str(url[len(url) - 2]).replace(group_name, '{}')
        url[1] = '{}'
        url_pattern = '/'.join(url)
        return group_name, url_pattern


    def _get_config(self):
        if (self.config is None):
            self.config = Config()

        return self.config


class Project:
    project = ''
    group = ''

    def __init__(self, project, group):
        self.project = str(project)
        self.group = str(group)
